package survey_result

import (
	"surveyService/response"
	"surveyService/service/survey_mechanism"
	"surveyService/service/survey_result"
	"surveyService/service/survey_token"
	"surveyService/util"
	"surveyService/util/constants"
	"surveyService/validators"

	"github.com/gin-gonic/gin"
	"gogs.uu.mdfitnesscao.com/hys/sdk"
)

// 创建问卷答题结果
func CreateSurveyResultForMember(c *gin.Context) {
	type Validator struct {
		ID string `json:"id" form:"id" binding:"required"`
	}
	var validator Validator
	if validateErr := c.ShouldBind(&validator); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)

	surveyToken, _ := util.GetFromGinContext[string](c, constants.SurveyTokenCacheKey)
	surveyTokenData, _ := util.GetFromGinContext[*validators.SurveyToken](c, constants.SurveyTokenDataCacheKey)
	surveyId := validator.ID
	extra := ""
	if surveyTokenData != nil {
		surveyId = surveyTokenData.SurveyId
		extra = surveyTokenData.Extra
	}

	surveyResult, err := survey_result.InitMember(member, mechanismId).Create(surveyId, extra)
	if err != nil {
		response.Fail(c, err)
		return
	}

	surveyQuestions, err := survey_mechanism.InitMember(member, mechanismId).DetailSurveyQuestions(surveyId)
	if err != nil {
		response.Fail(c, err)
		return
	}
	if surveyTokenData != nil {
		// 将结果编号更新到Token里
		survey_token.RenewSurveyToken(surveyToken, &validators.SurveyToken{
			SurveyId:       surveyTokenData.SurveyId,
			SurveyResultId: surveyResult.SN,
			Extra:          surveyTokenData.Extra,
			MechanismId:    surveyTokenData.MechanismId,
		}, 7200)
	}

	response.Success(c, map[string]any{
		"detail":    survey_result.Format(surveyResult, true),
		"questions": surveyQuestions,
	})
}

// 提交问卷答题结果
func SubmitSurveyResultForMember(c *gin.Context) {
	type Validator struct {
		ID     string         `json:"id" form:"id" binding:"required"`
		Answer map[string]any `json:"answer" form:"answer" binding:"required"`
	}
	var validator Validator
	if validateErr := c.ShouldBind(&validator); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)
	surveyTokenData, _ := util.GetFromGinContext[*validators.SurveyToken](c, constants.SurveyTokenDataCacheKey)
	surveyResultId := validator.ID
	if surveyTokenData != nil {
		surveyResultId = surveyTokenData.SurveyResultId
	}

	err := survey_result.InitMember(member, mechanismId).Submit(surveyResultId, validator.Answer)
	if err != nil {
		response.Fail(c, err)
		return
	}

	response.Success(c, map[string]any{})
}

// 创建并提交问卷答题结果
func CreateAndSubmitSurveyResultForMember(c *gin.Context) {
	type Validator struct {
		ID        string         `json:"id" form:"id" binding:"required"`
		Answer    map[string]any `json:"answer" form:"answer" binding:"required"`
		StartTime string         `json:"startTime" form:"startTime" binding:"required"`
	}
	var validator Validator
	if validateErr := c.ShouldBind(&validator); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)

	surveyToken, _ := util.GetFromGinContext[string](c, constants.SurveyTokenCacheKey)
	surveyTokenData, _ := util.GetFromGinContext[*validators.SurveyToken](c, constants.SurveyTokenDataCacheKey)
	surveyId := validator.ID
	extra := ""
	if surveyTokenData != nil {
		surveyId = surveyTokenData.SurveyId
		extra = surveyTokenData.Extra
	}

	surveyResult, err := survey_result.InitMember(member, mechanismId).CreateAndSubmit(surveyId, validator.Answer, validator.StartTime, extra)
	if err != nil {
		response.Fail(c, err)
		return
	}

	if surveyTokenData != nil {
		// 将结果编号更新到Token里
		survey_token.RenewSurveyToken(surveyToken, &validators.SurveyToken{
			SurveyId:       surveyTokenData.SurveyId,
			SurveyResultId: surveyResult.SN,
			Extra:          surveyTokenData.Extra,
			MechanismId:    surveyTokenData.MechanismId,
		}, 7200)
	}

	response.Success(c, map[string]any{
		"id": surveyResult.SN,
	})
}

// 获取问卷答题结果
func DetailSurveyResultForMember(c *gin.Context) {
	type Validator struct {
		ID string `json:"id" form:"id" binding:"required"`
	}
	var validator Validator
	if validateErr := c.ShouldBindQuery(&validator); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)
	surveyTokenData, _ := util.GetFromGinContext[*validators.SurveyToken](c, constants.SurveyTokenDataCacheKey)
	surveyResultId := validator.ID
	if surveyTokenData != nil {
		surveyResultId = surveyTokenData.SurveyResultId
	}

	surveyResult, err := survey_result.InitMember(member, mechanismId).Detail(surveyResultId)
	if err != nil {
		response.Fail(c, err)
		return
	}

	response.Success(c, map[string]any{
		"detail": survey_result.Format(surveyResult, true),
	})
}

// 获取问卷答题结果
func DetailLastSurveyResultForMember(c *gin.Context) {
	type Validator struct {
		SurveyId         int64 `json:"surveyId" form:"surveyId" binding:"omitempty"`
		IsPhysicalSurvey int64 `json:"isPhysicalSurvey" form:"isPhysicalSurvey" binding:"omitempty"`
	}
	var validator Validator
	if validateErr := c.ShouldBindQuery(&validator); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	if validator.IsPhysicalSurvey != 1 && validator.SurveyId == 0 {
		response.Fail(c, &response.ErrCode{
			Code: response.ERROR,
			Msg:  "问卷编号是必填的",
		})
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)

	surveyResult, err := survey_result.InitMember(member, mechanismId).DetailLastSurvey(validator.SurveyId, validator.IsPhysicalSurvey == 1)
	if err != nil {
		response.Fail(c, err)
		return
	}

	response.Success(c, map[string]any{
		"detail": survey_result.Format(surveyResult, true),
	})
}

// 获取问卷结果列表
func PaginateForMember(c *gin.Context) {
	type Validator struct {
		Page     int `json:"page" form:"page" binding:"omitempty,min=1"`
		PageSize int `json:"pageSize" form:"pageSize" binding:"omitempty,min=1"`
	}
	var v Validator
	if validateErr := c.ShouldBindQuery(&v); validateErr != nil {
		response.FailValidator(c, validateErr)
		return
	}
	mechanismId, _ := util.GetFromGinContext[string](c, constants.MemberMechanismIDKey)
	member, _ := util.GetFromGinContext[*sdk.AuthMember](c, constants.MemberCacheKey)
	surveyResults, total := survey_result.InitMember(member, mechanismId).Paginate(v.Page, v.PageSize)
	var list []*validators.SurveyResult = make([]*validators.SurveyResult, 0)
	for _, surveyResult := range surveyResults {
		list = append(list, survey_result.Format(surveyResult, false))
	}
	response.Success(c, map[string]any{
		"list":  list,
		"total": total,
	})

}
